#include "snot.h"

/*
 * parse_ip - take a string of ip address information and convert it to a linked list
 * 
 * Input:
 *
 *  ip: ip address string
 *  addr_list: pointer to an address list structure to receive the addresses
 *  num: pointer to an int that receives the total number of converted addresses
 *  line: line number
 *
 * Returns:
 *
 *  0: Success
 *  1: Failure
 *
 */

int parse_ip(char *ip, struct addr_list **al, int *num, int line)
{
	char *tempstr,*temp, *temp2, *ipstring;
	char *tok;
	struct hostent *host;
	struct in_addr *ina;
#ifdef DEBUG
	struct in_addr pina;
#endif
	unsigned long mask;
	struct addr_list *alcurr;
	
	*num = 0;

	
	tempstr = strdup(ip);
	temp=tempstr;
	
	/* Clear list */
	*al = NULL;

	if(*temp == (char)'[')
	{
		temp++;
	}
	
	do{
		if (*temp == ',')
		{
			temp++;
		}
		while(isspace(*temp))
		{
			temp++;
		}
		if((temp2 = strpbrk(temp, ",]")) != NULL)
		{
			if((ipstring = malloc(temp2-temp+1)) == NULL)
			{
				printf("parse_ip: malloc failed..\n");
				return(1);
			}
			memset(ipstring,0,(temp2-temp+1));
			ipstring = strncpy(ipstring, temp, (temp2-temp));
			
		}
		else
		{
			ipstring = strdup(temp);
		}
		temp2 = ipstring;

		/* Give us something to work with */
		if((alcurr = malloc(sizeof(struct addr_list))) == NULL)
		{
			printf("parse_ip: malloc failed..\n");
			return(1);
		}

		if(*temp2 == (char)'!')
		{
#ifdef DEBUG
			printf("found NOT\n");
#endif
			alcurr->addrnot = 1;
			temp2++;
		}
		else
		{
			alcurr->addrnot = 0;
		}
		
		if(*temp2 == (char)'$')
		{
			/* Chances are this person hasn't set their variables properly */
			printf("parse_ip: [line %d]: Did you set your variables correctly? (parsing %s)\n",line,ip);
			return(1);
		}
		
		if (!(STRCASECMP(temp2,"any")))
		{
			alcurr->addrmask = netmask[0];
			alcurr->addr = 0;
		}
		else
		{
			if((tok = strchr(temp2, '/'))!= NULL)
			{
				/* We have a netmask */
				*tok = (char)NULL;
				tok++;
				mask = atoi(tok);
				if(mask<=32)
				{
					alcurr->addrmask = netmask[mask];
#ifdef DEBUG
					pina.s_addr = alcurr->addrmask;
					printf("parse_ip: Address mask is %s\n",inet_ntoa(pina));
#endif
				}
				else
				{
					printf("parse_ip: Netmask too phat.\n");
					return(1);
				}
			}
			else
			{
#ifdef DEBUG
				printf("parse_ip: Host address\n");
#endif
				/* No '/' ? Then its a host address */
				alcurr->addrmask = 0xFFFFFFFF;
			}
			/* Now lookup the IP address */
			if (inet_addr (temp2) == -1)
			{
				/* Try Resolving it */
#ifdef DEBUG
				printf("temp2 = %s\n",temp2);
#endif
				if ((host = gethostbyname (temp2)) == NULL)
				{
					printf("parse_ip: [line %d]: Couldn't resolve address %s\n",line,temp2);
					return(1);
				}
				else
				{
					ina = (struct in_addr *) host->h_addr_list[0];
#ifdef DEBUG
					printf ("DNS IP : %s\n", inet_ntoa (*ina));
#endif
					alcurr->addr = ina->s_addr;
#ifdef DEBUG
					printf ("DNS Hostname : %s\n", ipstring);
#endif
				}
			}
			else
			{
				alcurr->addr = inet_addr (temp2);
			}
		}			
		alcurr->next = *al;
		*al = alcurr;
		free(ipstring);
		*num = *num + 1;
	}
	while((temp = strpbrk(temp,",")) != NULL);
		
	free(tempstr);
	return(0);
}

/* end parse_ip */


/*
 * expand_vars - expand a rule line with applicable variables
 * 
 * Input:
 *
 *  rule: a snort rule string
 *  line: current line number
 *
 * Returns:
 *
 *  0: Success
 *  1: Failure
 *
 */

int expand_vars(char **rule,int line)
{
	char *temp,*temp2,*temp3;
	int length = 0;
	struct var_list *vtemp;

	if((temp = malloc(RULE_MAX)) == NULL)
	{
		printf("expand_vars: [line %d] Couldnt malloc..\n",line);
		return(1);
	}
	
	memset(temp, 0, RULE_MAX);

	temp2 = *rule;
	temp3 = temp;
	while ((*temp2) != (char)NULL)
	{
		/* Have we got a variable? */
		if(*temp2 == '$')
		{
			/* Skip the $ */
			temp2++;
			if ((*temp2) == (char)NULL)
			{
				/* The $ is right at the end of the string.. paste it and move on */
				*temp3 = '$';
				free(*rule);
				*rule = temp;
				return(0);
			}
			else
			{
				/* check all vars */
				vtemp = vhead;
				while((vtemp != NULL) && (strncmp(vtemp->varname, temp2, strlen(vtemp->varname))))
				{
					vtemp = vtemp->next;
				}
				/* Check if we matched or just reached the end of the rules list */
				if(vtemp == NULL)
				{
					*temp3 = '$';
					temp3++;
				}
				else
				{
					if(strlen(vtemp->value)+(temp3-temp) >= RULE_MAX)
					{
						printf("expand_vars: [line %d] Variable expansion creates string > %d, ignoring..\n",line, RULE_MAX);
						free(temp);
						return(1);
					}
					strcat(temp3, vtemp->value);
					temp3 = temp3 + strlen(vtemp->value);
					temp2 = temp2 + strlen(vtemp->varname);
				}
			}
		}
		else
		{
			/* Check if we are going to blow the end of the malloced string */
			if((temp3 - temp) == RULE_MAX)
			{
				printf("expand_vars: [line %d] Variable expansion creates string > %d, ignoring..\n",line, RULE_MAX);
				free(temp);
				return(1);
			}
			/* Just a normal character */
			*temp3 = *temp2;
			temp2++;
			temp3++;
		}
	}
	free(*rule);
	*rule = temp;
	return(0);
}
/* end expand_vars */


/*
 * parse_rules: parse a snort rules file
 * 
 * Input:
 *
 *  globsrc: pointer to global overridden source ip address string
 *  globdst: pointer to global overridden destination ip address string
 *
 * Returns:
 *
 *  0: Success
 *  1: Failure
 *
 */

int parse_rules(char *globsrc, char *globdst, char *rulefile)
{
	char *token, *tok2;
	char *rs, *rulestring, *ruletemp;
	char *optstring;
	FILE *f;
	int line,i,j,found,found2;
	int pmin, pmax, offset, depth, contsize;
	char *arg, *temp, *temp2;
	int *flags;
	int hex1,hex2;
	unsigned char *uc;
	char *ia;
	char *ds;
	unsigned short *us;
	unsigned long *ul;
	int *mi;

	struct var_list *vtemp,*vcurrent,*vprev;
	struct rule *r;

	struct option_list
	{
		char *option_string;
		struct option_list *next;
	};
	
	struct option_list *olhead, *olcur, *oltemp;
	
	/* Set up the netmask table */
	netmask[0]=inet_addr("0.0.0.0");
	netmask[1]=inet_addr("128.0.0.0");
	netmask[2]=inet_addr("192.0.0.0");
	netmask[3]=inet_addr("224.0.0.0");
	netmask[4]=inet_addr("240.0.0.0");
	netmask[5]=inet_addr("248.0.0.0");
	netmask[6]=inet_addr("252.0.0.0");
	netmask[7]=inet_addr("254.0.0.0");
	netmask[8]=inet_addr("255.0.0.0");
	netmask[9]=inet_addr("255.128.0.0");
	netmask[10]=inet_addr("255.192.0.0");
	netmask[11]=inet_addr("255.224.0.0");
	netmask[12]=inet_addr("255.240.0.0");
	netmask[13]=inet_addr("255.248.0.0");
	netmask[14]=inet_addr("255.252.0.0");
	netmask[15]=inet_addr("255.254.0.0");
	netmask[16]=inet_addr("255.255.0.0");
	netmask[17]=inet_addr("255.255.128.0");
	netmask[18]=inet_addr("255.255.192.0");
	netmask[19]=inet_addr("255.255.224.0");
	netmask[20]=inet_addr("255.255.240.0");
	netmask[21]=inet_addr("255.255.248.0");
	netmask[22]=inet_addr("255.255.252.0");
	netmask[23]=inet_addr("255.255.254.0");
	netmask[24]=inet_addr("255.255.255.0");
	netmask[25]=inet_addr("255.255.255.128");
	netmask[26]=inet_addr("255.255.255.192");
	netmask[27]=inet_addr("255.255.255.224");
	netmask[28]=inet_addr("255.255.255.240");
	netmask[29]=inet_addr("255.255.255.248");
	netmask[30]=inet_addr("255.255.255.252");
	netmask[31]=inet_addr("255.255.255.254");
	netmask[32]=inet_addr("255.255.255.255");

	line = total_rules = 0;
	
	/* Arbitrarily large number for size of the rule line */
	if ((rs = malloc(RULE_MAX)) == NULL)
	{
		printf("parse_rules: [line %d] : malloc failed..\n",line);
		return(1);
	}
#ifdef DEBUG
	printf("rule file is %s\n",rulefile);
#endif
	if((f = fopen(rulefile,"r"))!=NULL)
	{
				
		/* Clear out our linked lists */
		rulehead = NULL;	
		vhead = NULL;

		while((fgets(rs, RULE_MAX-1, f)) != NULL)
		{
			line++;
			/* check for dos chars in the rulestring */
			if((ruletemp = strchr(rs, 0x0d))||(ruletemp = strchr(rs, 0x0a)))
			{
				*ruletemp = 0;
			}
			rulestring = strdup(rs);
			ruletemp = rulestring;

			/* is it a comment or a blank line? */
			if ((rulestring) && (*rulestring) && (isalnum(*rulestring)))
			{
				rulestring = ruletemp;
				if(expand_vars(&rulestring,line))
				{
					return(1);
				}
				/* rulestring has been freed and malloced in expand vars */
				ruletemp = rulestring;
				
				optstring = strdup(rulestring);
#ifdef DEBUG
				printf("rulestring = %s\n", rulestring);
#endif
				while(isspace(*rulestring))
				{
					rulestring++;
				}
				/* variable line? */
				if(!(STRNICMP(rulestring, "var", 3)))
				{
					if ((token = strtok(rulestring, " ")) != NULL)
					{
						if ((token = strtok(NULL, " ")) != NULL)
						{
							if((vtemp = malloc(sizeof(struct var_list))) == NULL)
							{
								printf("parse_rules: [line %d] : malloc failed.\n",line);
								return(1);
							}
							vtemp->varname = strdup(token);
							arg = token;
							while(((*arg) != (char)NULL) &&(!isspace(*arg)))
							{
								arg++;
							}
							arg++;
							if(strlen(arg) == 0)
							{
								printf("parse_rules: [line %d]: Invalid Variable Value, ignoring\n",line);
								free(vtemp);
								goto ignore;
							}
							vtemp->value = strdup(arg);

							/* Put the variable in the linked list in alphabetical order */
							vprev = NULL;
							vcurrent = vhead;
							while((vcurrent != NULL) && (vcurrent->value != NULL) && (strncmp(vtemp->varname, vcurrent->varname, strlen(vcurrent->varname)) >= 0))
							{
								vprev = vcurrent;
								vcurrent = vcurrent->next;
							}
							if (vprev != NULL)
							{
								vtemp->next = vprev->next;
								vprev->next = vtemp;
							}
							else
							{
								vtemp->next = vhead;
								vhead = vtemp;
							}

#ifdef DEBUG
								
							print_var(vtemp);
#endif
						}
						else
						{
							printf("parse_rules: [line %d]: Invalid Variable Name, ignoring\n",line);
							goto ignore;
						}
					}
					else
					{
						printf("parse_rules: [line %d]: Invalid Variable Name, ignoring\n",line);
						goto ignore;
					}
				}			
				else if ((token = strtok(rulestring, " ")) != NULL)
				{
					
					/* Give us a new rule to work with */

					if((r = malloc(sizeof(struct rule))) == NULL)
					{
						printf("parse_rules: [line %d] : malloc failed.\n",line);
						return(1);
					}

					/* log or alert rule? */
					if((!(STRCASECMP(token, "alert")))||(!(STRCASECMP(token, "log"))))
					{
						/* get the protocol type */
						if ((token = strtok(NULL, " ")) != NULL)
						{
							if(!(STRCASECMP(token, "tcp")))
							{
								r->proto = TCP;
							}
							else if(!(STRCASECMP(token, "udp")))
							{
								r->proto = UDP;
							}
							else if(!(STRCASECMP(token, "icmp")))
							{
								r->proto = ICMP;
							}
							else
							{
								printf("parse_rules: [line %d]: Protocol Error, ignoring\n",line);
								goto ignore;
							}	
						}
						/* get the source IP */
						
						/* first check if its been overridden */
						if (globsrc != NULL)
						{
							if(parse_ip(globsrc,&(r->srcaddr), &(r->srcnum), line))
							{
								goto ignore;
							}
							if ((token = strtok(NULL, " ")) == NULL)
							{
								printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
								goto ignore;
							}
						}
						else if ((token = strtok(NULL, " ")) != NULL)
						{		
							if(parse_ip(token,&(r->srcaddr),&(r->srcnum),line))
							{
								goto ignore;
							}
						}
						else
						{
							printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
							goto ignore;
						}
						/* get the source port number */
						if ((token = strtok(NULL, " ")) != NULL)
						{
							if(!(STRCASECMP(token, "any")))
							{
								r->srcport = ANY;
							}
							else
							{
								if(*token == '!')
								{
									r->srcportnot = 1;
									token++;
								}
								else
								{
									r->srcportnot = 0;
								}

								r->srcport = atoi(token);
							}
						}
						else
						{
							printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
							goto ignore;
						}
						/* get the direction */
						if ((token = strtok(NULL, " ")) != NULL)
						{
							if((!(STRCASECMP(token, "->")))||(!(STRCASECMP(token, "<>"))))
							{
								if(!(STRCASECMP(token, "->")))
								{
									r->bidirectional = FALSE;
								}
								else
								{
									r->bidirectional = TRUE;
								}
							}
						}
						else
						{
							printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
							goto ignore;
						}
						/* get the dest IP */
						
						/* first check if its been overridden */
						if (globdst != NULL)
						{
							if(parse_ip(globdst,&(r->destaddr),&(r->destnum),line))
							{
								goto ignore;
							}
							if ((token = strtok(NULL, " ")) == NULL)
							{
								printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
								goto ignore;
							}
						}
						else if ((token = strtok(NULL, " ")) != NULL)
						{
							if(parse_ip(token,&(r->destaddr),&(r->destnum),line))
							{
								goto ignore;
							}				
						}
						else
						{
							printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
							goto ignore;
						}
						/* get the dest port number */
						if ((token = strtok(NULL, " ")) != NULL)
						{
							if(!(STRCASECMP(token, "any")))
							{
								r->destport = ANY;
							}
							else
							{		
								if(*token == '!')
								{
									r->destportnot = 1;
									token++;
								}
								else
								{
									r->destportnot = 0;
								}

								r->destport = atoi(token);
							}
						}
						else
						{
							printf("parse_rules: [line %d]: Truncated, ignoring\n",line);
							goto ignore;
						}
						
						if (r->proto == 0)
						{
							printf("parse_rules: [line %d]: Invalid Rule, ignoring\n",line);
							goto ignore;
						}
						
						
						
						/* now work out the crap at the end */
						if((token = strtok(optstring, "(")) != NULL)
						{
/*							if((olhead = malloc(sizeof(struct option_list))) == NULL)
							{
								printf("parse_rules: [line %d]: malloc failed\n", line);
								return(1);
							}
							*/
							olhead = NULL;
							
/*							current = olhead;
							current->option_string = NULL;
							current->next = NULL; */
							/* Make sure there is at least one option in here */
							if ((token = strtok(NULL, ";")) == NULL)
							{
								printf("parse_rules: [line %d]: Malformed options string, ignoring\n",line);
								goto ignore;
							}
							
							/* skip any spaces */
							while(isspace(*token)) 
							{
								token++;
								if(*token == '\0')
								{
									printf("parse_rules: [line %d]: Malformed options string, ignoring\n",line);
									goto ignore;
								}
							}
							/* Loop through all the options, and make a linked list of them */
							while((token != NULL)&&(*token != ')'))
							{
								if((olcur = malloc(sizeof(struct option_list))) == NULL)
								{
									printf("parse_rules: [line %d]: malloc failed\n", line);
									return(1);
								}
								/* token is a option field now */
								olcur->option_string = strdup(token);	
								if ((token = strtok(NULL, ";")) != NULL)
								{				
									/* skip any spaces */
									while(isspace(*token)) 
									{
										token++;
										if(*token == '\0')
										{
											printf("parse_rules: [Line %d]: Malformed options string, ignoring\n",line);
											goto ignore;
										}
									}
									/* Stick it on the front of the list */
									olcur->next = olhead;
									olhead = olcur;
/*									
									if((current->next = malloc(sizeof(struct option_list)))==NULL)
									{
										printf("parse_rules: [line %d]: malloc failed\n", line);
										return(1);
									}
									current = current->next;
									*/
								}
								else
								{
									printf("parse_rules: [line %d]: Missing ; terminator in options string, ignoring\n",line);
									goto ignore;
								}
							}
/*							current->option_string = NULL;
							current->next = NULL;
							*/
							olcur = olhead;	
							
							/* Blank out the array */
							memset(r->optionlist, 0, (sizeof(char *) * OPTION_MAX));	
							
							/* Search the list for our option strings, and make the option array */
							while(olcur != NULL)
							{
								if(!(STRCASECMP("nocase",olcur->option_string)))
								{
									/* special case for the only option with no argument */
									r->optionlist[NOCASE] = (char *)1;
								}
								else if((arg = strchr(olcur->option_string, ':')) != NULL)					
								{
									token = strtok(olcur->option_string, ":");
									arg++;
									while(isspace(*arg)) 
									{
										arg++;
									}
									if(*arg == '\0')
									{
										printf("parse_rules: [line %d]: Malformed options string, ignoring\n",line);
										goto ignore;
									}
									/* if there is a trailing bracket, chop it off */
									/*									if(temp = strrchr(arg, ')'))
									{
									*temp = '\0';
									}
									*/									/* arg now points to the argument */
									found = 0;
									for(i=0;(i!=OPTION_MAX);i++)
									{
										if(!(STRCASECMP(option_text[i],token)))
										{
											found = 1;
											if(r->optionlist[i] != NULL)
											{
											/* some tool has duped the option strings, just take
												the last one */
												free(r->optionlist[i]);
											}
											/* special code to handle each type of option */
											switch (i)
											{
											case MSG:
												r->optionlist[i] = strdup(arg);
												break;
												/* These take u_char */
											case ITYPE:
											case ICODE:
											case TTL:
											case TOS:
												if ( (unsigned char)atoi(arg) > 255 )
												{
													printf("parse_rules: [line %d]: option %s greater than 255, ignoring\n",line, option_text[i]);
													goto ignore;
												}
												if ((uc = malloc(sizeof(unsigned char))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												*uc = (unsigned char)atoi(arg);
												
												r->optionlist[i] = uc;
												
												break;
												/* These take u_short */
											case ICMP_ID:
											case ICMP_SEQ:
											case ID:
												if ( atoi(arg) > 65535 )
												{
													printf("parse_rules: [line %d]: option %s greater than allowed size 65,535, ignoring\n",line, option_text[i]);
													goto ignore;
												}
												if ((us = malloc(sizeof(unsigned short))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												*us = atoi(arg);
												
												r->optionlist[i] = (char *)us;
												
												break;
												/* These take u_int_32 */
											case SEQ:
											case ACK:
												if ((ul = malloc(sizeof(unsigned long))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												*ul = (unsigned long)atol(arg);
												
												r->optionlist[i] = (char *)ul;
												
											case DEPTH:
											case OFFSET:
												if ((mi = malloc(sizeof(int))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												*mi = atoi(arg);
												r->optionlist[i] = (char *)mi;
												break;
											case IPOPTS:
												
												if((ia = malloc(IPOPTS_MAX)) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												
												memset(ia, FALSE, IPOPTS_MAX);
												while((tok2 = strtok(NULL, ",")) != NULL)
												{
													while(isspace(*tok2)) 
													{
														tok2++;
													}
													if(*tok2 == '\0')
													{
														printf("parse_rules: [line %d]: Malformed ipopts string, ignoring\n",line);
														goto ignore;
													}			
													temp = tok2;
													/* Chop trailing spaces */
													while(*temp != '\0')
													{
														if(isspace(*temp))
														{
															*temp = '\0';
														}
														temp++;
													}
													
													found2 = FALSE;
													for(j=0;(j!=IPOPTS_MAX);j++)
													{
														if(!(STRCASECMP(ipopts_text[j],tok2)))
														{
															ia[j] = TRUE;
															found2 = TRUE;
														}
													}
													
													if (found2 == FALSE)
													{
														printf("parse_rules: [line %d]: Illegal ipopts string %s, ignoring\n", line, tok2);
														goto ignore;
													}
												}
												
												r->optionlist[i] = ia;
												break;
											case DSIZE:
												
												if((ds = malloc(sizeof(struct dsize))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												
												if(*arg == '<')
												{
													((struct dsize *)ds)->gtlt = DSIZELT;
													arg++;
												}
												else if(*arg == '>')
												{
													((struct dsize *)ds)->gtlt = DSIZEGT;
													arg++;
												}
												else
													// This is an exact match
												{
													((struct dsize *)ds)->gtlt = DSIZEEQ;
												}
												((struct dsize *)ds)->size = atoi(arg);
												if (((struct dsize *)ds)->size > (1500-(LIBNET_PACKET)))
												{
													printf("parse_rules: [line %d]: dsize too large, ignoring\n",line);
													goto ignore;
												}
												r->optionlist[i] = ds;
												
												break;
											case FRAGBITS:
												while(isspace(*arg)) 
												{
													arg++;
												}
												if((flags = malloc(sizeof(int))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												*flags = 0;
												while(*arg != '\0')
												{
													
													switch(*arg)
													{
													case 'r':
													case 'R':
														*flags |= FRAGR;
														break;
													case 'd':
													case 'D':
														*flags |= FRAGD;
														break;
													case 'm':
													case 'M':
														*flags |= FRAGM;
														break;
													case '*':
														*flags |= FRAGSTAR;
														break;
													case '+':
														*flags |= FRAGPLUS;
														break;
													case '-':
														*flags |= FRAGMINUS;
														break;
														
													default:
														printf("parse_rules: [line %d]: Illegal fragmentation flags char \"%c\", ignoring\n", line, *arg);
														goto ignore;
														break;
													}
													arg++;
												}
												if(*flags == 0)
												{
													printf("parse_rules: [line %d]: No fragmentation flags provided, ignoring\n",line);
													goto ignore;
												}
												r->optionlist[i] = (char *)flags;
												break;
											case FLAGS:
												while(isspace(*arg)) 
												{
													arg++;
												}
												if((flags = malloc(sizeof(int))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n");
													return(1);
												}
												
												*flags = 0;
												while(*arg != '\0')
												{
													
													switch(*arg)
													{
													case 'f':
													case 'F':
														*flags |= TFIN;
														break;
													case 's':
													case 'S':
														*flags |= TSYN;
														break;
													case 'r':
													case 'R':
														*flags |= TRST;
														break;
													case 'p':
													case 'P':
														*flags |= TPSH;
														break;
													case 'a':
													case 'A':
														*flags |= TACK;
														break;
													case 'u':
													case 'U':
														*flags |= TURG;
														break;
													case '2':
														*flags |= TRES2;
														break;
													case '1':
														*flags |= TRES1;
														break;
													case '+':
														*flags |= TPLUS;
														break;
													case '*':
														*flags |= TSTAR;
														break;
													case '-':
														*flags |= TMINUS;
														break;
													case '0':
														*flags = TZERO;
														break;
														
													default:
														printf("parse_rules: [line %d]: Illegal tcpflags char \"%c\", ignoring\n", line, *arg);
														goto ignore;
														break;
													}
													arg++;
												}
												
												if(*flags == 0)
												{
													printf("parse_rules: [line %d]: No tcp flags provided, ignoring\n",line);
													goto ignore;
												}
												
												r->optionlist[i] = (char *)flags;
												
												break;
												
											case CONTENT:
												/* First skip past all the spaces */
												while(isspace(*arg)) 
												{
													arg++;
												}
												/* First char must be the open quote */
												if(*arg != '\"')
												{
													printf("parse_rules: [line %d]: Missing open quote on content string, ignoring\n",line);
													goto ignore;
												}


												if((r->optionlist[i] = malloc(sizeof(struct content))) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n",line);
												}
												/* If our content string is bigger than 1500 bytes, we have a problem */
												if((temp = malloc(1500)) == NULL)
												{
													printf("parse_rules: [line %d]: malloc failed..\n",line);
													return(1);
												}
												temp2 = temp;
												
												/* Make a nice blank space to work with */
												memset(temp,0,1500);

												/* Kill leading spaces */
												while(isspace(*arg))
												{
													arg++;
												}
												/* Trim opening quote */
												if (*arg == '\"')
												{
													arg++;
												}
												else
												{
													printf("parse_rules: [line %d]: Missing open quote on content string, ignoring\n",line);
													goto ignore;
												}
												/* Loop till we reach the close quote */
												while(*arg != '\"')
												{
													if(*arg == '\0')
													{
														printf("parse_rules: [line %d]: Missing close quote on content string, ignoring\n",line);
														goto ignore;
													}
													
													if(*arg == '|')
													{
														arg++;
														while(*arg != '|')
														{
															while(isspace(*arg))
															{
																arg++;
															}
															
															if((*arg >= 'A' && *arg <= 'F') || (*arg >= 'a' && *arg <= 'f') || (*arg >= '0' && *arg <= '9'))
															{
																/* Calculate the value of this hex char */
																if((*arg >= 'A' && *arg <= 'F') || (*arg >= 'a' && *arg <= 'f'))
																{
																	hex1 = tolower(*arg);
																	hex2 = ((hex1 - 'a') + 10) * 16;
																}
																else
																{
																	hex2 = (*arg - '0') * 16;
																}
																arg++;
																if(!((*arg >= 'A' && *arg <= 'F') || (*arg >= 'a' && *arg <= 'f') || (*arg >= '0' && *arg <= '9')))							
																{
																	/* Dont allow those stupid odd sized hex chars - ambiguity in network byte ordering marty */
																	if((isspace(*arg))||(*arg == '|'))
																	{
																		printf("parse_rules: [line %d]: Hex char too short in content string, ignoring\n",line);
																		
																	}
																	else
																	{
																		printf("parse_rules: [line %d]: Invalid char \"%c\" found in hex component of content string, ignoring\n",line,*temp);
																	}
																	free(temp2);
																	goto ignore;
																}
																if((*arg >= 'A' && *arg <= 'F') || (*arg >= 'a' && *arg <= 'f'))
																{
																	hex1 = tolower(*arg);
																	hex2 = hex2 + ((hex1 - 'a') + 10);
																}
																else
																{
																	hex2 = hex2 + (*arg - '0');
																}
																*temp = hex2;
																arg++;
																temp++;
																
																if((temp - temp2) == 1500)
																{
																	printf("parse_rules: [line %d]: Content string too large.. ignoring\n",line);
																	free(temp2);
																	goto ignore;
																}
															}
															else
															{
																printf("parse_rules: [line %d]: Invalid char \"%c\" found in hex component of content string, ignoring\n",line,*temp);
																free(temp2);
																goto ignore;
															}
														}
													}
													else
													{
														*temp = *arg;
														/* Its just a normal char so it goes straight into the buffer */
														temp++;
													}
													if((temp - temp2) == 1500)
													{
														printf("parse_rules: [line %d]: Content string too large.. ignoring\n",line);
														goto ignore;
													}
													arg++;
												}
												if ((((struct content *)r->optionlist[i])->pcont = malloc(temp-temp2)) == NULL)
												{
													printf("parse_rules: [line %d]: Couldn't malloc..\n",line);
													return(1);
												}
												memcpy(((struct content *)r->optionlist[i])->pcont,temp2,temp-temp2);
												((struct content *)r->optionlist[i])->size = (temp - temp2);
												free(temp2);
												
												break;
											case CONTENT_LIST:
											case RPC:
											case REFERENCE:
												break;
											default:
												printf("parse_rules: [line %d]: !! Cant get here.. parse_rules.c\n");
												return(1);
											}
										}
									}
									if (found == 0)
									{
										printf("parse_rules: [line %d]: Unknown Option \"%s\", ignoring option\n", line, token);
									}
								
								}
								else
								{
									printf("parse_rules: [line %d]: Illegal Option String, ignoring\n",line);
									goto ignore;
								}	
						
								oltemp = olcur->next;
								free(olcur->option_string);
								free(olcur);
								olcur = oltemp;
							}
						}

#ifdef DEBUG
						print_rule(r);
#endif
						/* Post rule process size sanity checks */

						pmin = 0;
						pmax = (1500-(LIBNET_PACKET));
						
						if (r->optionlist[OFFSET] != NULL)
						{
							offset = *(r->optionlist[OFFSET]);
						}
						else
						{
							offset = 0;
						}	
						if (r->optionlist[DEPTH] != NULL)
						{
							depth = *(r->optionlist[DEPTH]);
						}
						else
						{
							depth = 0;
						}
						if (r->optionlist[CONTENT] != NULL)
						{
							contsize = ((struct content *)r->optionlist[CONTENT])->size;
						}
						else
						{
							contsize = 0;
						}
						
						if ((r->optionlist[DSIZE]) != NULL)
						{
							if(((struct dsize *)r->optionlist[DSIZE])->gtlt == DSIZEGT)
							{
								pmin = ((struct dsize *)r->optionlist[DSIZE])->size;
							}
							else if(((struct dsize *)r->optionlist[DSIZE])->gtlt == DSIZELT)
							{
								pmax = ((struct dsize *)r->optionlist[DSIZE])->size;	
							}
							else
							{
								pmin = pmax = ((struct dsize *)r->optionlist[DSIZE])->size;
							}
						}
						if (depth > pmax)
						{
							printf("parse_rules: [line %d]: depth is greater than the dsize, ignoring\n",line);
							goto ignore;
						}
						if (offset > pmax)
						{
							printf("parse_rules: [line %d]: offset is greater than the dsize, ignoring\n",line);
							goto ignore;
						}
						
						if (pmax < (offset + depth))
						{
							printf("parse_rules: [line %d]: conflict between offset, depth and dsize, ignoring\n",line);
							goto ignore;
						}
						if (pmax < (offset + contsize))
						{
							printf("parse_rules: [line %d]: conflict between offset, content size and dsize, ignoring\n",line);
							goto ignore;
						}
						
						/* Put the rule at the head of the linked list */
						r->next = rulehead;
						rulehead = r;

						/* Successfully parsed! */
						total_rules++;
ignore:
						;
					}
				}
				else
				{
					printf("parse_rules: [line %d]: \"%s\" token not handled, ignoring..\n",line);
				}
				free(optstring);
			}
			free(ruletemp);
		}
	}
	else
	{
		perror("fopen");
		return(1);
	}
	return(0);
}

/* end parse_rules */


/*
 * print_var: print out a variable
 * 
 * Input:
 *
 *  var_list: a variable entry
 *
 */

void print_var(struct var_list *var)
{
	printf("==================\n");
	printf("Var Name        : %s\n",var->varname);
	printf("Var Value       : %s\n",var->value);
	printf("==================\n");
}

/* end print_var */


/*
 * print_rule: print out a stored rule
 * 
 * Input:
 *
 *  rulepnt: a rule structure
 *
 */

void print_rule(struct rule *rulepnt)
{
	int i,j;
	char *temp;
	struct in_addr ina;
	struct addr_list *addrlist;

	printf("==================\n");
	printf("Rule Protocol   : ");
	if(rulepnt->proto == TCP) printf("TCP\n");
	else if(rulepnt->proto == UDP) printf("UDP\n");
	else if(rulepnt->proto == ICMP) printf("ICMP\n");

	i = rulepnt->srcnum;
	addrlist = rulepnt->srcaddr;
	while(i)
	{
		i--;
		ina.s_addr = addrlist->addr;
		printf("Rule Source Addr: ");
		if(addrlist->addrnot) printf("NOT ");
		printf("%s\n",inet_ntoa(ina));
		ina.s_addr = addrlist->addrmask;
		printf("Rule Source Mask: %s\n",inet_ntoa(ina));
		addrlist=addrlist->next;
	}

	printf("Rule Source Port: ");
	if(rulepnt->srcportnot) printf("NOT ");

	if (rulepnt->srcport == ANY)
	{
		printf("ANY\n");
	}
	else				
	{
		printf("%d\n",rulepnt->srcport);
	}
	i = rulepnt->destnum;
	addrlist = rulepnt->destaddr;
	while(i)
	{
		i--;
		ina.s_addr = addrlist->addr;
		printf("Rule Dest Addr: ");
		if(addrlist->addrnot) printf("NOT ");
		printf("%s\n",inet_ntoa(ina));
		ina.s_addr = addrlist->addrmask;
		printf("Rule Dest Mask: %s\n",inet_ntoa(ina));
		addrlist=addrlist->next;
	}

	printf("Rule Dest Port  : ");
	if(rulepnt->destportnot) printf("NOT ");
	if (rulepnt->destport == ANY)
	{
		printf("ANY\n");
	}
	else				
	{
		printf("%d\n",rulepnt->destport);
	}
	printf("Rule Bidir      : ");
	if (rulepnt->bidirectional == TRUE)
	{
		printf("TRUE\n");
	}
	else 
	{
		printf("FALSE\n");
	}
	for(i=0;(i!=OPTION_MAX);i++)
	{
		if(rulepnt->optionlist[i] != NULL)
		{
			switch(i)
			{
				case IPOPTS:
					for(j=0; (j!=IPOPTS_MAX);j++)
						if((rulepnt->optionlist[i][j]) != FALSE)
						{
							printf("IP Option: %s\n",ipopts_text[j]);
						}
					break;
				case DSIZE:
					printf("Dsize: ");
					if((((struct dsize *)rulepnt->optionlist[i])->gtlt) == 2)
					{
						printf("<");
					}
					else if((((struct dsize *)rulepnt->optionlist[i])->gtlt) == 1)
					{
						printf(">");
					}
					printf("%d\n",(((struct dsize *)rulepnt->optionlist[i])->size));
					break;
				case FLAGS:
					printf("TCP Flags: ");
					if((int)*(rulepnt->optionlist[i]) & TZERO)
					{
						printf("0");
					}
					else
					{
						if((int)*(rulepnt->optionlist[i]) & TFIN) printf("F");
						if((int)*(rulepnt->optionlist[i]) & TSYN) printf("S");
						if((int)*(rulepnt->optionlist[i]) & TRST) printf("R");
						if((int)*(rulepnt->optionlist[i]) & TPSH) printf("P");
						if((int)*(rulepnt->optionlist[i]) & TACK) printf("A");
						if((int)*(rulepnt->optionlist[i]) & TURG) printf("U");
						if((int)*(rulepnt->optionlist[i]) & TRES2) printf("2");
						if((int)*(rulepnt->optionlist[i]) & TRES1) printf("1");
						if((int)*(rulepnt->optionlist[i]) & TPLUS) printf("+");
						if((int)*(rulepnt->optionlist[i]) & TSTAR) printf("*");
						if((int)*(rulepnt->optionlist[i]) & TMINUS) printf("-");
					}
					printf("\n");
					break;
				case FRAGBITS:
					printf("Frag Bits: ");
					if(*(rulepnt->optionlist[i]) & FRAGR) printf("R");
					if(*(rulepnt->optionlist[i]) & FRAGD) printf("D");
					if(*(rulepnt->optionlist[i]) & FRAGM) printf("M");
					if(*(rulepnt->optionlist[i]) & FRAGPLUS) printf("+");
					if(*(rulepnt->optionlist[i]) & FRAGSTAR) printf("*");
					if(*(rulepnt->optionlist[i]) & FRAGMINUS) printf("-");
					printf("\n");
					break;
				case NOCASE:
					printf("Nocase - not implemented\n");
					break;
				case MSG:
					printf("Message         : %s\n", rulepnt->optionlist[i]);
					break;
				case RPC:
					printf("RPC - not implemented\n");
					break;
				case ID:
				case SEQ:
				case ACK:

				case ITYPE:
				case ICODE:
				case TTL:
				case TOS:
					// unsigned char
					break;
				case ICMP_ID:
				case ICMP_SEQ:
					printf("ICMP %s: %d\n", option_text[i], *(rulepnt->optionlist[i]));
					break;
				case DEPTH:
					printf("Depth: %d\n",*(rulepnt->optionlist[i]));
					break;
				case OFFSET:
					printf("Offset %d\n", *(rulepnt->optionlist[i]));
					break;
				case CONTENT:
					temp = rulepnt->optionlist[i];
					temp++;
					printf("Content String: ");
					for(j=0; j<(*(rulepnt->optionlist[i])); j++)
					{

						printf("%02x ",*temp);
						temp++;
					}
					printf("\n");
					break;

				default:
					printf("option %s, arg %s\n",option_text[i], rulepnt->optionlist[i]);
					break;
			}
		}
	}
	printf("==================\n\n");
}

/* end print_rule */