#include "snot.h"

/*
 * send_packets - Send the packets to the opened interface
 * 
 * Input:
 *
 *  mdly: integer maximum number of seconds delay
 *  mnum: integer maximum number of packets to send, 0 = infinite
 *
 * Returns:
 *
 *  0: Success
 *  1: Failure
 *
 */

int send_packets(int mdly, int mnum)
{
	int sport;
	int dport;
	unsigned long sad;
	unsigned long dad;
	unsigned long tad;
	char tadstr[16];	/* string big enough for an ip address */
	
	unsigned char tos;
	unsigned short id;
	unsigned short frag;
	unsigned char ttl;
	
	unsigned long seq;
	unsigned long ack;
	unsigned char flags;
	
	char *payload;
	unsigned char *packet;
	int payloadsize, iphdrsize;
	int s, sock, pmin, pmax, offset, depth, contsize, rf, delay ,j ,k, numleft;
	unsigned short i;
	struct in_addr ina;
	struct libnet_icmp_hdr icmp_hdr;
	struct ipoption ipo;

	unsigned char ip_options[MAX_IPOPTLEN] = { '0' };	/* decent size array to handle our ip options */

	int pickaddr;
	struct addr_list *addrlist;

	struct rule *rulecurrent,**rulearray;


#if(WIN32)
	/* Stuff for the waitable timer */
    HANDLE hTimer = NULL;
    LARGE_INTEGER liDueTime;
	
	// Create a waitable timer.
    hTimer = CreateWaitableTimer(NULL, TRUE, "snotTimer");
    if (!hTimer)
    {
        printf("CreateWaitableTimer failed (%d)\n", GetLastError());
        return 1;
    }

#endif

	srand( (unsigned)time( NULL ) );
		
	if ((sock=libnet_open_raw_sock(IPPROTO_RAW))==-11)
	{
		printf("send_packets: couldn't open raw socket\n");
		return(1);
	}

	if ((payload = malloc(LIBNET_MAX_PACKET)) == NULL)
	{
		printf("send_packets: malloc failed..\n");
		return(1);
	}

	/* Build a rule array - quicker to index */

	if ((rulearray = malloc(total_rules * sizeof(char *))) == NULL)
	{
		printf("send_packets: malloc failed..\n");
		return(1);
	}

	rulecurrent = rulehead;
	j = 0;
	while(rulecurrent != NULL)
	{
		rulearray[j] = rulecurrent;
		j++;
		rulecurrent = rulecurrent->next;
	}

	numleft = mnum;

	while((mnum == 0)||(numleft))
	{

		numleft--;

		/* Pick a random rule */
		rulecurrent = rulearray[rand()%total_rules];

		/* Its an IP header to begin with */
		iphdrsize = LIBNET_IP_H;

		/* Do the random addresses */
		
		/* Pick a source addr */

		addrlist = rulecurrent->srcaddr;

		pickaddr = rand()%(rulecurrent->srcnum);
		while(pickaddr)
		{
			addrlist = addrlist->next;
			pickaddr--;
		}

		/* Is it a host address? */
		if(addrlist->addrmask != 0xFFFFFFFF)
		{
			sprintf(tadstr, "%u.%u.%u.%u",(rand()%239),(rand()%254),(rand()%254),(rand()%254));
			tad = inet_addr(tadstr);
			if(addrlist->addrnot && addrlist->addrmask == 0)
			{
				printf("Error: Can't generate source '!' address for netmask = 0\n");
				return(1);
			}
			if(addrlist->addrnot)
			{
				/* Check that the network isnt the same - not much chance but just in case */
				while((addrlist->addr & addrlist->addrmask)==(tad & addrlist->addrmask))
				{
					sprintf(tadstr, "%u.%u.%u.%u",(rand()%239),(rand()%254),(rand()%254),(rand()%254));
					tad = inet_addr(tadstr);
				}
				/* Completely random should do the trick */
				sad = tad;
			}
			else
			{			
				/* Blank off the bits that are network addresses */
				tad &= (0xFFFFFFFF - addrlist->addrmask);
				/* Make sure its not a network address or broadcast address */
				while ((tad == 0)||(tad == (0xFFFFFFFF - addrlist->addrmask)))
				{
					sprintf(tadstr, "%u.%u.%u.%u",(rand()%239),(rand()%254),(rand()%254),(rand()%254));
					tad = inet_addr(tadstr);
					tad &= (0xFFFFFFFF - addrlist->addrmask);
				}
				/* OR the mask and the source network together to blank the host bits, then add the */
				/* random host bits in there */
				sad = (addrlist->addr & addrlist->addrmask) | tad;
			}
		}
		else
		{
			sad = (addrlist->addr);
		}
		/* Destination addr */
		addrlist = rulecurrent->destaddr;

		pickaddr = rand()%(rulecurrent->destnum);
		while(pickaddr)
		{
			addrlist = addrlist->next;
			pickaddr--;
		}

		/* Is it a host address? */
		if(addrlist->addrmask != 0xFFFFFFFF)
		{
			sprintf(tadstr, "%u.%u.%u.%u",(rand()%239),(rand()%254),(rand()%254),(rand()%254));
			tad = inet_addr(tadstr);
			if(addrlist->addrnot && addrlist->addrmask == 0)
			{
				printf("Error: Can't generate destination '!' address for netmask = 0\n");
				return(1);
			}
			if(addrlist->addrnot)
			{
				/* Check that the network isnt the same - not much chance but just in case */
				while((addrlist->addr & addrlist->addrmask)==(tad & addrlist->addrmask))
				{
					sprintf(tadstr, "%u.%u.%u.%u",(rand()%239),(rand()%254),(rand()%254),(rand()%254));
					tad = inet_addr(tadstr);
				}
				/* Completely random should do the trick */
				dad = tad;
			}
			else
			{
				/* Blank off the bits that are network addresses */
				tad &= (0xFFFFFFFF - addrlist->addrmask);
				/* Make sure its not a network address or broadcast address */
				while ((tad == 0)||(tad == (0xFFFFFFFF - addrlist->addrmask)))
				{
					sprintf(tadstr, "%u.%u.%u.%u",(rand()%239),(rand()%254),(rand()%254),(rand()%254));
					tad = inet_addr(tadstr);
					tad &= (0xFFFFFFFF - addrlist->addrmask);
				}
				/* OR the mask and the dest network together to blank the host bits, then add the */
				/* random host bits in there */
				dad = (addrlist->addr & addrlist->addrmask) | tad;
			}
		}
		else
		{
			dad = (addrlist->addr);
		}

#ifdef DEBUG
		ina.s_addr = sad;
		printf("Source Address = %s\n",inet_ntoa(ina));
		ina.s_addr = dad;
		printf("Dest Address = %s\n",inet_ntoa(ina));
#endif		
		/* Type of service */
		if (rulecurrent->optionlist[TOS] == NULL)
		{
			tos = rand()%sizeof(tos);
		}
		else
		{
			tos = (unsigned char)*rulecurrent->optionlist[TOS];
		}
		
		/* Packet ID */
		if (rulecurrent->optionlist[ID] == NULL)
		{
			id = (unsigned short)rand();
		}
		else
		{
			id = *((unsigned short *)rulecurrent->optionlist[ID]);
		}
		/* Fragmentation Bits */
		
		frag = 0;
		if ((rulecurrent->optionlist[FRAGBITS]) != NULL)
		{
			if((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGPLUS) 
			{
				if((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGR) frag |= IP_RF;
				if((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGD) frag |= IP_DF;
				if((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGM) frag |= IP_MF;
			}
			else if((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGSTAR)
			{
				if(((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGR)&&((rand()%2)-1)) frag |= IP_RF;
				if(((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGD)&&((rand()%2)-1)) frag |= IP_DF;
				if(((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGM)&&((rand()%2)-1)) frag |= IP_MF;
			}
			else if((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGMINUS)
			{
				if((!((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGR))&&((rand()%2)-1)) frag |= IP_RF;
				if((!((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGD))&&((rand()%2)-1)) frag |= IP_DF;
				if((!((int)*(rulecurrent->optionlist[FRAGBITS]) & FRAGM))&&((rand()%2)-1)) frag |= IP_MF;
			}
		}
		
		/* TTL */
		if((rulecurrent->optionlist[TTL]) != NULL)
		{
			ttl = *(rulecurrent->optionlist[TTL]);
		}
		else
		{
			ttl = (unsigned char)rand();
		}
		
		/* IP Options */

		k = 0;
		if((rulecurrent->optionlist[IPOPTS]) != NULL)
		{
			for(j=0; (j!=IPOPTS_MAX);j++)
			{
				if((rulecurrent->optionlist[IPOPTS][j]) != FALSE)
				{
					switch (j)
					{
					case RR:
						ip_options[k++] = 7;
						ip_options[k++] = 7;
						ip_options[k++] = 4;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						break;
					case EOL:
						ip_options[k++] = 0;
						break;
					case NOP:
						ip_options[k++] = 1;
						break;
					case TS:
						ip_options[k++] = 68;
						ip_options[k++] = 10;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						break;
					case SEC:
						ip_options[k++] = 130;
						ip_options[k++] = 11;
						/* security */
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						/* compartments */
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						/* handling */
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						/* tcc */
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						break;
						/* Not sure where marty got these two from, its not in rfc 791 */
					case LSRR:
						ip_options[k++] = 131;
						ip_options[k++] = 7;
						ip_options[k++] = 4;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						break;
					case LSRRE:
						ip_options[k++] = 132;
						ip_options[k++] = 7;
						ip_options[k++] = 4;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						break;
					case SSRR:
						/* Going to predict the source route? I think not */
						break;
					case SATID:
						ip_options[k++] = 136;
						ip_options[k++] = 4;
						ip_options[k++] = rand()%255;
						ip_options[k++] = rand()%255;
						break;
					}
				}
			}

			ip_options[k++] = 0;
			while(k%4)
			{
				ip_options[k++] = 0;
			}
			memcpy(ipo.ipopt_list, ip_options, k);
		}
		
		/* Payload */

		pmin = 0;
		pmax = (1500-(LIBNET_PACKET));

		if (rulecurrent->optionlist[OFFSET] != NULL)
		{
			offset = *(rulecurrent->optionlist[OFFSET]);
		}
		else
		{
			offset = 0;
		}	
		if (rulecurrent->optionlist[DEPTH] != NULL)
		{
			depth = *(rulecurrent->optionlist[DEPTH]);
		}
		else
		{
			depth = 0;
		}
		if (rulecurrent->optionlist[CONTENT] != NULL)
		{
			contsize = ((struct content *)rulecurrent->optionlist[CONTENT])->size;
		}
		else
		{
			contsize = 0;
		}

		if ((rulecurrent->optionlist[DSIZE]) != NULL)
		{
			if(((struct dsize *)rulecurrent->optionlist[DSIZE])->gtlt == DSIZEGT)
			{
				pmin = ((struct dsize *)rulecurrent->optionlist[DSIZE])->size;
			}
			else if(((struct dsize *)rulecurrent->optionlist[DSIZE])->gtlt == DSIZELT)
			{
				pmax = ((struct dsize *)rulecurrent->optionlist[DSIZE])->size;	
			}
			else
			{
				pmin = pmax = ((struct dsize *)rulecurrent->optionlist[DSIZE])->size;
			}
		}

#ifdef DEBUG
		printf("pmax = %d, pmin = %d\n",pmax,pmin);
#endif
		if (pmin == pmax)
		{
			payloadsize = pmin;
		}
		else
		{
			payloadsize = pmin + (rand()%(pmax-pmin));
		}

		// Fill the payload with random crap
		for(i=0;i<payloadsize;i++)
		{
			payload[i]=rand()%255;
		}
		 
		if (contsize)
		{	
			pmin = offset;
			if (depth)
			{
				pmax = offset+depth-1;
			}
			else
			{
				pmax = payloadsize - contsize;
			}
			
			if( pmin == pmax )
			{
				i = pmin;
			}
			else
			{
				i = pmin + (rand()%(pmax-pmin));
			}
#ifdef DEBUG
			printf("pasting content at %d\n",i);
#endif
			/* pick a random number between pmin and pmax, and paste the content there */
			memcpy(payload + i, ((struct content *)rulecurrent->optionlist[CONTENT])->pcont, contsize);

		}	

		if ((libnet_init_packet(LIBNET_MAX_PACKET,&packet))!=1)
		{
#ifdef DEBUG
			printf("iphdrsize = %d, payloadsize = %d\n",iphdrsize,payloadsize);
#endif
			printf("send_packets: Packet memory allocation failed\n");
			return(1);
		}


		/* Split off and do the packet types */
		
		if(rulecurrent->proto == UDP)
		{
						
			if ((libnet_build_ip((unsigned short)payloadsize+LIBNET_UDP_H, 
				(unsigned char)tos, 
				(unsigned short)id, 
				(unsigned short)htons(frag),
				(unsigned char)ttl,
				IPPROTO_UDP,
				(unsigned long)sad,
				(unsigned long)dad,
				packet+iphdrsize, 
				(int)payloadsize+LIBNET_UDP_H, 
				(unsigned char *)packet)) == -1)
			{
				printf("libnet_build_ip failed\n");
			}

			if (k)
			{
				libnet_insert_ipo(&ipo, k, packet); 
			}

			iphdrsize = (((struct libnet_ip_hdr *)packet)->ip_hl) * 4;

			/* Do the ports */
			if(rulecurrent->srcport != ANY)
			{
				if(rulecurrent->srcportnot)
				{
					while((sport=(rand()%65535))== rulecurrent->srcport)
						;
				}
				else
				{
					sport = rulecurrent->srcport;
				}
			}
			else
			{
				sport = (unsigned short)rand();
			}
			
			if(rulecurrent->destport != ANY)
			{
				if(rulecurrent->destportnot)
				{
					while((dport=(rand()%65535))== rulecurrent->destport)
						;
				}
				else
				{
					dport = rulecurrent->destport;
				}
			}
			else
			{
				dport = (unsigned short)rand();
			}
			
			if ((libnet_build_udp((u_short)sport, 
				(u_short) dport, 
				payload, 
				payloadsize, 
				packet+iphdrsize)) == -1)
			{
				printf("Build UDP packet failed\n");
				return(1);
			}
			
			if (libnet_do_checksum (packet, IPPROTO_UDP, LIBNET_UDP_H+payloadsize) == -1)
			{
				printf("UDP checksum failed\n");
				return(1);
			}
			if (libnet_do_checksum (packet, IPPROTO_IP, iphdrsize) == -1)
			{
				printf("IP checksum failed\n");
				return(1);
			}
			
#ifdef DEBUG
			printf("sending packet with payloadsize = %d\n",iphdrsize+payloadsize);
#endif
			printf("UDP - %s",rulecurrent->optionlist[MSG]);
			ina.s_addr = sad;
			printf(" - %s:%d ", inet_ntoa(ina),sport);
			ina.s_addr = dad;
			printf("-> %s:%d\n", inet_ntoa(ina),dport);
			if ((s = libnet_write_ip (sock, packet, iphdrsize+LIBNET_UDP_H+payloadsize)) == -1)
			{
				perror("send_packets: libnet_write_ip failed");
			}
			
		}
		else if(rulecurrent->proto == TCP)
		{

			if ((libnet_build_ip((unsigned short)payloadsize+LIBNET_TCP_H, 
				(unsigned char)tos, 
				(unsigned short)id, 
				(unsigned short)htons(frag),
				(unsigned char)ttl,
				IPPROTO_TCP,
				(unsigned long)sad,
				(unsigned long)dad,
				NULL,
				0,
				(unsigned char *)packet)) == -1)
			{
				printf("libnet_build_ip failed\n");
			}

			if (k)
			{
				libnet_insert_ipo(&ipo, k, packet); 
			}

			iphdrsize = (((struct libnet_ip_hdr *)packet)->ip_hl) * 4;

			/* Do the ports */
			if(rulecurrent->srcport != ANY)
			{
				if(rulecurrent->srcportnot)
				{
					while((sport=(rand()%65535))== rulecurrent->srcport)
						;
				}
				else
				{
					sport = rulecurrent->srcport;
				}
			}
			else
			{
				sport = (unsigned short)rand();
			}
			
			if(rulecurrent->destport != ANY)
			{
				if(rulecurrent->destportnot)
				{
					while((dport=(rand()%65535))== rulecurrent->destport)
						;
				}
				else
				{
					dport = rulecurrent->destport;
				}
			}
			else
			{
				dport = (unsigned short)rand();
			}
			
			/* Sequence numbers */
			if(rulecurrent->optionlist[SEQ])
			{
				seq = (unsigned long)rulecurrent->optionlist[SEQ];
			}
			else
			{
				seq = rand()*rand();
			}
			if(rulecurrent->optionlist[ACK])
			{
				ack = (unsigned long)rulecurrent->optionlist[ACK];
			}
			else
			{
				ack = rand()*rand();
			}
			
			/* TCP Flags */
			flags = 0;
			/* If no tcp flags were set, just treat the rule like it was a star with all of the flags set */
			if (rulecurrent->optionlist[FLAGS] == NULL) 
			{
				rf = (int)(TSTAR|TFIN|TSYN|TRST|TPSH|TACK|TURG|TRES1|TRES2);
			}
			else
			{
				rf = (int)*(rulecurrent->optionlist[FLAGS]);
			}
			if (rf != TZERO)
			{
				if(rf & TSTAR)
				{
					if((rf & TFIN)&&((rand()%2)-1)) flags |= TH_FIN;
					if((rf & TSYN)&&((rand()%2)-1)) flags |= TH_SYN;
					if((rf & TRST)&&((rand()%2)-1)) flags |= TH_RST;
					if((rf & TPSH)&&((rand()%2)-1)) flags |= TH_PSH;
					if((rf & TACK)&&((rand()%2)-1)) flags |= TH_ACK;
					if((rf & TURG)&&((rand()%2)-1)) flags |= TH_URG;
					if((rf & TRES2)&&((rand()%2)-1)) flags |= TH_RES2;
					if((rf & TRES1)&&((rand()%2)-1)) flags |= TH_RES1;
				}
				else if(rf & TMINUS)
				{
					if((!(rf & TFIN))&&((rand()%2)-1)) flags |= TH_FIN;
					if((!(rf & TSYN))&&((rand()%2)-1)) flags |= TH_SYN;
					if((!(rf & TRST))&&((rand()%2)-1)) flags |= TH_RST;
					if((!(rf & TPSH))&&((rand()%2)-1)) flags |= TH_PSH;
					if((!(rf & TACK))&&((rand()%2)-1)) flags |= TH_ACK;
					if((!(rf & TURG))&&((rand()%2)-1)) flags |= TH_URG;
					if((!(rf & TRES2))&&((rand()%2)-1)) flags |= TH_RES2;
					if((!(rf & TRES1))&&((rand()%2)-1)) flags |= TH_RES1;
				}
				else /* This covers the + case and the case where no +*- is specified */
				{
					if(rf & TFIN) flags |= TH_FIN;
					if(rf & TSYN) flags |= TH_SYN;
					if(rf & TRST) flags |= TH_RST;
					if(rf & TPSH) flags |= TH_PSH;
					if(rf & TACK) flags |= TH_ACK;
					if(rf & TURG) flags |= TH_URG;
					if(rf & TRES2) flags |= TH_RES2;
					if(rf & TRES1) flags |= TH_RES1;
				}
			}
			
			if ((libnet_build_tcp((u_short)sport,
				(u_short) dport,
				seq,
				ack,
				flags,
				(u_short)rand(),	/* Window size */
				(u_short)rand(),	/* Urgent Pointer */
				payload,
				payloadsize,
				packet + iphdrsize)) == -1)
			{
				printf("Build TCP packet failed\n");
				return(1);
			}

			if (libnet_do_checksum (packet, IPPROTO_TCP, LIBNET_TCP_H+payloadsize) == -1)
			{
				printf("TCP checksum failed\n");
				return(1);
			}
			if (libnet_do_checksum (packet, IPPROTO_IP, iphdrsize) == -1)
			{
				printf("IP checksum failed\n");
				return(1);
			}
#ifdef DEBUG
			printf("sending packet with payloadsize = %d\n",iphdrsize+payloadsize);
#endif
			printf("TCP - %s",rulecurrent->optionlist[MSG]);
			ina.s_addr = sad;
			printf(" - %s:%d ", inet_ntoa(ina),sport);
			ina.s_addr = dad;
			printf("-> %s:%d\n", inet_ntoa(ina),dport);
			
			if ((s = libnet_write_ip (sock, packet, iphdrsize+LIBNET_TCP_H+payloadsize)) == -1)
			{
				perror("send_packets: libnet_write_ip failed");
			}
			
		}
		else if(rulecurrent->proto == ICMP)
		{
		

			if ((libnet_build_ip((unsigned short)payloadsize+LIBNET_ICMP_ECHO_H, 
				(unsigned char)tos, 
				(unsigned short)id, 
				(unsigned short)htons(frag),
				(unsigned char)ttl,
				IPPROTO_ICMP,
				(unsigned long)sad,
				(unsigned long)dad,
				packet+iphdrsize, 
				(int)payloadsize+LIBNET_ICMP_ECHO_H,
				(unsigned char *)packet)) == -1)
			{
				printf("libnet_build_icmp failed\n");
			}

			if (k)
			{
				libnet_insert_ipo(&ipo, k, packet); 
			}

			iphdrsize = (((struct libnet_ip_hdr *)packet)->ip_hl) * 4;

			/* Do the ports */
			if(rulecurrent->srcport != ANY)
			{
				sport = rulecurrent->srcport;
			}
			else
			{
				sport = (unsigned short)rand();
			}
			
			if(rulecurrent->destport != ANY)
			{
				dport = rulecurrent->destport;
			}
			else
			{
				dport = (unsigned short)rand();
			}
			
			/* ICMP settings */
			if(rulecurrent->optionlist[ITYPE] != NULL)
			{
				icmp_hdr.icmp_type = *(rulecurrent->optionlist[ITYPE]);	/* packet type */
			}
			else
			{
				icmp_hdr.icmp_type = (unsigned char)rand();
			}
			if(rulecurrent->optionlist[ICODE] != NULL)
			{
				icmp_hdr.icmp_code = *(rulecurrent->optionlist[ICODE]);	/* packet code */
			}
			else
			{
				icmp_hdr.icmp_code = (unsigned char)rand();
			}
			if(rulecurrent->optionlist[ICMP_ID] != NULL)
			{
				icmp_hdr.icmp_id = *(rulecurrent->optionlist[ICMP_ID]);	/* packet id */
			}
			else
			{
				icmp_hdr.icmp_id = (unsigned short)rand();
			}
			if(rulecurrent->optionlist[ICMP_SEQ] != NULL)
			{
				icmp_hdr.icmp_seq = *(rulecurrent->optionlist[ICMP_SEQ]);	/* packet seq */
			}
			else
			{
				icmp_hdr.icmp_seq = (unsigned short)rand();
			}
	        memcpy(packet+iphdrsize+LIBNET_ICMP_ECHO_H, payload, payloadsize);
		    memcpy(packet+iphdrsize, &icmp_hdr, LIBNET_ICMP_ECHO_H);	/* Includes ECHO size due to it covering all the fields we need */

			if (libnet_do_checksum (packet, IPPROTO_ICMP, LIBNET_ICMP_ECHO_H+payloadsize) == -1)
			{
				printf("ICMP checksum failed\n");
				return(1);
			}
			if (libnet_do_checksum (packet, IPPROTO_IP, iphdrsize) == -1)
			{
				printf("IP checksum failed\n");
				return(1);
			}
#ifdef DEBUG
			printf("sending packet with payloadsize = %d\n",iphdrsize+payloadsize);
#endif
			printf("ICMP - %s",rulecurrent->optionlist[MSG]);
			ina.s_addr = sad;
			printf(" - %s ", inet_ntoa(ina));
			ina.s_addr = dad;
			printf("-> %s\n", inet_ntoa(ina));
			if ((s = libnet_write_ip (sock, packet, iphdrsize+LIBNET_ICMP_ECHO_H+payloadsize)) == -1)
			{
				perror("send_packets: libnet_write_ip failed");
			}
		}

		libnet_destroy_packet(&packet);

		if(mdly)
		{
			delay = rand()%mdly;
			printf("Sleeping for %d seconds\n",delay);
#if(WIN32)	
			liDueTime.QuadPart=10000000*(delay)*(-1);
			
			if (!SetWaitableTimer(
				hTimer, &liDueTime, 0, NULL, NULL, 0))
			{
				printf("SetWaitableTimer failed (%d)\n", GetLastError());
				return(1);
			}
			
			// Wait for the timer.
			if (WaitForSingleObject(hTimer, INFINITE) != WAIT_OBJECT_0)
			{
				printf("WaitForSingleObject failed (%d)\n", GetLastError());
				return(1);
			}
			
#else
			sleep(delay);
#endif
		}
	}
	return(0);
}

/* end send_packets */